@@ -1,10 +1,10 @@
 package Job::Machine::DB;
-$Job::Machine::DB::VERSION = '0.19';
+$Job::Machine::DB::VERSION = '0.20';
 use strict;
 use warnings;
 use Carp qw/croak confess/;
 use DBI;
-use JSON;
+use Data::Serializer;
 
 use constant QUEUE_PREFIX    => 'jm:';
 use constant RESPONSE_PREFIX => 'jmr:';
@@ -14,17 +14,20 @@ sub new {
 	croak "No connect information" unless $args{dbh} or $args{dsn};
 	croak "invalid queue" if ref $args{queue} and ref $args{queue} ne 'ARRAY';
 
+	$args{dbh_inherited} = 1 if $args{dbh};
 	$args{user}     ||= undef;
 	$args{password} ||= undef;
 	$args{db_attr}  ||= undef;
 	$args{dbh}      ||= DBI->connect($args{dsn},$args{user},$args{password},$args{db_attr});
-	$args{schema}   ||= 'jobmachine';
+	$args{database_schema}   ||= 'jobmachine';
 	return bless \%args, $class;
 }
 
-sub json {
+sub serializer {
 	my ($self) = @_;
-	return $self->{json} ||= JSON->new->allow_nonref;
+	my $args = $self->{serializer_args} || {};
+	$args->{serializer} ||= $self->{serializer} // 'Sereal';
+	return $self->{serialize} ||= Data::Serializer->new(%$args);
 }
 
 sub listen {
@@ -86,14 +89,14 @@ sub fetch_work_task {
 	$self->{current_table} = 'task';
 	my $elems = join(',', ('?') x @$queue);
 	my $sql = qq{
-		UPDATE "$self->{schema}".$self->{current_table} t
+		UPDATE "$self->{database_schema}".$self->{current_table} t
 		SET status=100,
 			modified=default
 		FROM "jobmachine".class cx
 		WHERE t.class_id = cx.class_id
 		AND task_id = (
 			SELECT min(task_id)
-			FROM "$self->{schema}".$self->{current_table} t
+			FROM "$self->{database_schema}".$self->{current_table} t
 			JOIN "jobmachine".class c USING (class_id)
 			WHERE t.status=0
 			AND c.name IN ($elems)
@@ -110,7 +113,7 @@ sub fetch_work_task {
 	) || return;
 
 	$self->{task_id} = $task->{task_id};
-	$task->{data} = $self->_decode(delete $task->{parameters});
+	$task->{data} = $self->serializer->deserialize(delete $task->{parameters});
 	return $task;
 }
 
@@ -118,9 +121,9 @@ sub insert_task {
 	my ($self,$data,$queue) = @_;
 	my $class = $self->fetch_class($queue);
 	$self->{current_table} = 'task';
-	my $frozen = $self->json->encode($data);
+	my $frozen = $self->serializer->serialize($data);
 	my $sql = qq{
-		INSERT INTO "$self->{schema}".$self->{current_table}
+		INSERT INTO "$self->{database_schema}".$self->{current_table}
 			(class_id,parameters,status)
 		VALUES (?,?,?)
 		RETURNING task_id
@@ -133,7 +136,7 @@ sub set_task_status {
 	my $id = $self->task_id;
 	$self->{current_table} = 'task';
 	my $sql = qq{
-		UPDATE "$self->{schema}".$self->{current_table}
+		UPDATE "$self->{database_schema}".$self->{current_table}
 		SET status=?
 		WHERE task_id=?
 	};
@@ -145,7 +148,7 @@ sub fetch_class {
 	$self->{current_table} = 'class';
 	my $sql = qq{
 		SELECT *
-		FROM "$self->{schema}".$self->{current_table}
+		FROM "$self->{database_schema}".$self->{current_table}
 		WHERE name=?
 	};
 	return $self->select_first(sql => $sql,data => [$queue]) || $self->insert_class($queue);
@@ -154,7 +157,7 @@ sub fetch_class {
 sub insert_class {
 	my ($self,$queue) = @_;
 	my $sql = qq{
-		INSERT INTO "$self->{schema}".$self->{current_table}
+		INSERT INTO "$self->{database_schema}".$self->{current_table}
 			(name)
 		VALUES (?)
 		RETURNING class_id
@@ -165,9 +168,9 @@ sub insert_class {
 sub insert_result {
 	my ($self,$data,$queue) = @_;
 	$self->{current_table} = 'result';
-	my $frozen = $self->json->encode($data);
+	my $frozen = $self->serializer->serialize($data);
 	my $sql = qq{
-		INSERT INTO "$self->{schema}".$self->{current_table}
+		INSERT INTO "$self->{database_schema}".$self->{current_table}
 			(task_id,result)
 		VALUES (?,?)
 		RETURNING result_id
@@ -180,13 +183,13 @@ sub fetch_result {
 	$self->{current_table} = 'result';
 	my $sql = qq{
 		SELECT *
-		FROM "$self->{schema}".$self->{current_table}
+		FROM "$self->{database_schema}".$self->{current_table}
 		WHERE task_id=?
 		ORDER BY result_id DESC
 	};
 	my $result = $self->select_first(sql => $sql,data => [$id]) || return;
 
-	return $self->_decode($result->{result})->{data};
+	return $self->serializer->deserialize($result->{result})->{data};
 }
 
 sub fetch_results {
@@ -194,26 +197,13 @@ sub fetch_results {
 	$self->{current_table} = 'result';
 	my $sql = qq{
 		SELECT *
-		FROM "$self->{schema}".$self->{current_table}
+		FROM "$self->{database_schema}".$self->{current_table}
 		WHERE task_id=?
 		ORDER BY result_id DESC
 	};
 	my $results = $self->select_all(sql => $sql,data => [$id]) || return;
 
-	return [map { $self->_decode($_->{result}) } @{ $results } ];
-}
-
-sub _decode {
-	my ($self,$data) = @_;
-	my $resultdata;
-	eval {
-		$resultdata = $self->json->utf8(!utf8::is_utf8($resultdata))->decode($data);
-	};
-	 if ($@) {
-		warn $@;
-		return;
-	}
-	return $resultdata;
+	return [map { $self->serializer->deserialize($_->{result}) } @{ $results } ];
 }
 
 # 1. Find started tasks that have passed the time limit, most probably because 
@@ -225,7 +215,7 @@ sub revive_tasks {
 	$self->{current_table} = 'task';
 	my $status = 100;
 	my $sql = qq{
-		UPDATE "$self->{schema}".$self->{current_table}
+		UPDATE "$self->{database_schema}".$self->{current_table}
 		SET status=0
 		WHERE status=?
 		AND modified < now() - INTERVAL '$max seconds'
@@ -245,7 +235,7 @@ sub fail_tasks {
 	my $limit = 100;
 	my $sql = qq{
 		SELECT task_id
-		FROM "$self->{schema}".$self->{current_table}
+		FROM "$self->{database_schema}".$self->{current_table}
 		GROUP BY task_id
 		HAVING count(*)>?
 		LIMIT ?
@@ -257,7 +247,7 @@ sub fail_tasks {
 	$self->{current_table} = 'task';
 	my $status = 900;
 	$sql = qq{
-		UPDATE "$self->{schema}".$self->{current_table}
+		UPDATE "$self->{database_schema}".$self->{current_table}
 		SET status=?
 		WHERE task_id IN ($task_ids)
 	};
@@ -275,7 +265,7 @@ sub remove_tasks {
 	$self->{current_table} = 'task';
 	my $limit = 100;
 	my $sql = qq{
-		DELETE FROM "$self->{schema}".$self->{current_table}
+		DELETE FROM "$self->{database_schema}".$self->{current_table}
 		WHERE modified < now() - INTERVAL '$after days'
 	};
 	my $result = $self->do(sql => $sql,data => []);
@@ -366,7 +356,8 @@ sub disconnect {
 }
 
 sub DESTROY {
-	$_[0]->disconnect();
+	my $self = shift;
+	$self->disconnect() unless $self->{dbh_inherited};
 	return;
 }
 
@@ -384,7 +375,7 @@ Job::Machine::DB
 
 =head1 VERSION
 
-version 0.19
+version 0.20
 
 =head1 NAME
 
@@ -460,7 +451,7 @@ Kaare Rasmussen <kaare at cpan dot net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Kaare Rasmussen.
+This software is copyright (c) 2015 by Kaare Rasmussen.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
